--[[
This plug in calculates dps values based on the calculator written by
Toskk (http://druid.wikispaces.com/ToskksDPSGearMethod),
Tangedyn (http://tangedyn.9tribes.net/cat-dps/catdps-calc/)
using research gathered by many others.
--]]





local function print(x)
	if(type(x)=="table") then
		for k,v in pairs(x) do
			ChatFrame1:AddMessage(k.." = "..tostring(v))
		end
	else
		ChatFrame1:AddMessage(x)
	end
end

local Options = {
	Enemy ={
		Mitigation	= CatPoints:MitigationFromArmor(7700),--.15,
		Miss				= .09,
		Dodge				= .05,
	},
	ShredsPerMangle=3,
	BaseStat='ATTACKPOWER',

	derivedStats = {
		Malorne = function()
			local eq = CatPoints.Bonus:GetBonus("Malorne Harness", nil, 70)
			return eq
		end,
		Idol = function()
			return CatPoints:GetIdol()
		end,
		Haste = function()
			return (GetCombatRating(CR_HASTE_MELEE))
		end,
		Crit = function()
			return GetCritChance()/100
		end,
		Hit= function()
			return (GetCombatRating(CR_HIT_MELEE))
		end,
		Expertise = function()
			return (GetCombatRating(CR_WEAPON_SKILL))
		end,
		WeaponDamage = function()
			local _, _, _, _, dmg, _, _ = UnitDamage("player")
			return dmg
		end,
		AttackPower = function()
			local base, posBuff, negBuff = UnitAttackPower("player")
			return base+posBuff+negBuff
		end,
		AttackSpeed = function(player)
			return player.Haste/15.76/100
		end,
		DodgeReduction= function(player)
			return player.Expertise*52/82/2.5*.25/100
		end,
		Melee=function(player)
			return player.WeaponDamage + (768+player.AttackPower)/14
		end,
		MeleeCrit=function(player)
			return player.Melee*player.CritMultiplier
		end,
		MissReduction=function(player)
			return player.Hit*52/82/10/100
		end,
		CritMultiplier = function(player)
			local mult = (2 * (1 + (player.PredatoryInstincts * 0.02)))
			if(player.Diamond) then
				mult = mult *1.03
			end
			return mult
		end,
		MangleDamage=function(player)
			local sf = (1 + player.SavageFury * 0.1)
			local mangle= (player.Melee * 1.6 + 264)
			if(player.Idol == "IdoloftheWild") then
				mangle = mangle + 24
			end

			return mangle*sf
		end,
		MangleCritDamage=function(player)
			local mangle = player.MangleDamage

			return mangle*player.CritMultiplier
		end,
		MangleCost = function(player)
			return 45-player.Ferocity - ((player.Thunderheart>=2 and 5) or 0)
		end,
		ShredDamage=function(player)
			local shred= 1.3*(player.Melee * 2.25 + 405)
			if(player.Idol == "EverbloomIdol") then
				shred = shred + 88
			end
			--TraceValue("player.Nordrassil")
			if(player.Nordrassil >=4) then
				shred = shred + 75
			end

			return shred
		end,
		ShredCritDamage=function(player)
			local shred = player.ShredDamage

			return shred*player.CritMultiplier
		end,
		ShredCost = function(player)
			return 60-player.ShreddingAttacks*9
		end
	},
	derivedFunctions ={
		RipDamage = function(player, cp)
			local _apmult = .06*cp
			if(_apmult > .24) then
				_apmult = .24
			end

			damage = 1.3 * (148+281*cp + _apmult*player.AttackPower+(player.Idol=="IdolofFeralShadows" and 4*cp*6 or 0)) * (player.Thunderheart>=4 and 1.15 or 1)
			return damage
		end
	}
}



function Options:display()
	return GetShapeshiftForm(true)==3
end

function Options:applyBonus(bonuses, considerSet)

	local relevant = nil
	local s = self:new({})
	if(#bonuses == 0) then
		relevant = {}
	end

	setmetatable(bonuses, {__index = function(_, name) if(relevant) then relevant[name]=true end return 0 end})

	--AP
	s.AttackPower = s.AttackPower + (bonuses.ATTACKPOWER + bonuses.ATTACKPOWERFERAL) *( 1 + s.HeartoftheWild*.02 )

	local malorneStr = 0

	if(considerSet) then
		if(s.Malorne < 4 and s.Malorne + bonuses["Malorne Harness"] >= 4) then
			malorneStr = 30
		elseif(s.Malorne >= 4 and s.Malorne + bonuses["Malorne Harness"] < 4) then
			malorneStr = -30
		end

		s.Malorne = s.Malorne + bonuses["Malorne Harness"]
	end


	--STR
	s.AttackPower = s.AttackPower + ((bonuses.STR+malorneStr)*(1+s.SurvivaloftheFittest*.01)*2) *( 1 + s.HeartoftheWild*.02 )*(1+ .1*s.Kings)

	--AGI->AP
	--print(s.AttackPower)
	s.AttackPower = s.AttackPower + (bonuses.AGI*(1+s.SurvivaloftheFittest*.01)) *( 1 + s.HeartoftheWild*.02 )*(1+ .1*s.Kings)

	--print(s.AttackPower)
	--AGI->Crit
	s.Crit = s.Crit + (bonuses.AGI*(1+s.SurvivaloftheFittest*.01))/2500*(1+ .1*s.Kings)

	--print(s.Crit)
	--CR_CRIT->Crit
	s.Crit = s.Crit + (bonuses.CR_CRIT)*(52/82/14)/100

	--CatPoints:Print(s.AttackPower, s.Crit)
	s.Penetration = s.Penetration + bonuses.ARMOR_PENETRATION
	s.Hit = s.Hit + bonuses.CR_HIT
	s.Haste = s.Haste + bonuses.CR_HASTE
	s.WeaponDamage = s.WeaponDamage + bonuses.MELEEDMG
	s.Expertise = s.Expertise + bonuses.CR_EXPERTISE +bonuses.CR_WEAPON_FERAL

	if(considerSet) then
		s.Nordrassil = s.Nordrassil + bonuses["Nordrassil Harness"]
		s.Thunderheart = s.Thunderheart + bonuses["Thunderheart Harness"]
	end

	if(bonuses.Idol and bonuses.Idol ~= 0) then
		s.Idol = bonuses.Idol
	end

	return s,relevant
end



function Options:calculate(dbg)
	local enemy = self.Enemy
	local baseArmor = CatPoints:ArmorFromMitigation(enemy.Mitigation, self.Penetration)
	local actualMitigation = CatPoints:MitigationFromArmor(baseArmor)

	local actualMiss = enemy.Miss -self.MissReduction
	if(actualMiss < 0) then
		actualMiss = 0
	end

	local actualDodge = enemy.Dodge - self.DodgeReduction
	if(actualDodge < 0) then
		actualDodge = 0
	end

	local totalMiss = (actualMiss + actualDodge)
	local actualHit = 1- totalMiss

	local actualCrit = self.Crit
	if(actualCrit>.75) then
		actualCrit = .75
	end

	local hit = 1-actualMiss-actualCrit-actualDodge

	local yCrit = actualHit* actualCrit
	local yHit = actualHit * (1- actualCrit)

	local nCrit = yCrit / (yCrit+yHit)
	local nHit = 1-nCrit

	local avgShred = self.ShredsPerMangle*(self.ShredDamage*nHit+self.ShredCritDamage*nCrit)
	local avgMangle = self.MangleDamage*nHit+self.MangleCritDamage*nCrit

	local rip5 = self.RipDamage(5)
	local rip4 = self.RipDamage(4)
	local rip3 = self.RipDamage(3)
	local rip = rip5

	if(self.ShredsPerMangle == 2 ) then
		rip = nHit^3*rip3 + 3*(nCrit*nHit^2*rip4 + nCrit^2*nHit*rip5) + nCrit^3*rip5
	elseif(self.ShredsPerMangle == 3) then
		rip = nHit^4*rip4 + (1-nHit^4)*rip5
	end

	local misses = (1/(1-totalMiss) -1)

	local energy = (misses*self.MangleCost/5+self.MangleCost)
		+ self.ShredsPerMangle*(misses*self.ShredCost/5+self.ShredCost)
		+ 30*(misses+1)

	local cycle = energy*.1

	--TraceValue("cycle")

	local oocEnergy = (cycle/30) * ((self.MangleCost + self.ShredsPerMangle*self.ShredCost +30)/(self.ShredsPerMangle +2))
	local malorneEnergy = (self.ShredsPerMangle + 1 + (1 + self.AttackSpeed) * cycle) * 0.04 * 20

	--TraceValue("oocEnergy")
	--TraceValue("malorneEnergy")

	if(self.OmenofClarity > 0) then
		energy = energy - oocEnergy
	end

	if(self.Malorne >= 2) then
		energy = energy - malorneEnergy
	end

	--TraceValue(false,"energy")
	cycle = energy*.1


	local avgMelee = (1+self.AttackSpeed)*cycle*self.Melee*(self.CritMultiplier*actualCrit + (hit - 0.25) + 0.25 * 0.7)

	local dps =  ((avgMangle + avgShred + avgMelee) * (1 - actualMitigation) + rip) / cycle

	dps = dps*(self.Naturalist*.02+1)

	if(dbg) then
		debug.sethook()
	end

	return dps
end




local Player = CatPoints:CreateTable("CatDPS-2.3", Options)



